#pragma once

#include "Likelihood.h"

#if PLATFORM_WINDOWS
#include <Windows.h>
#else
#include <dlfcn.h>
#endif

extern "C" {
	typedef bool (*AllocateModelsFn)(size_t num_models);
	typedef void (*DeallocateModelsFn)();
	typedef void (*EvaluateLikelihoodFn)(size_t model_i, const Real* values, Real& llh);
	typedef void (*GetOutputFn)(size_t model_i, Real* output_values, size_t& numvalues);
}

class LikelihoodDynamicLibrary : public bcm::Likelihood
{
public:
	LikelihoodDynamicLibrary(size_t numthreads);
	virtual ~LikelihoodDynamicLibrary();

	bool Load(const std::string& filename);

	virtual bool EvaluateLogProbability(size_t threadix, const Real* values, Real& logp);
	virtual bool GetOutput(size_t threadix, Real* output_values, size_t& numvalues);

private:
	template<typename T>
	T GetFunction(const char* name);

	size_t numthreads;
	bool allocated;

#if PLATFORM_WINDOWS
	HMODULE module;
#else
	void* library;
#endif

	AllocateModelsFn allocate_models;
	DeallocateModelsFn deallocate_models;
	EvaluateLikelihoodFn evaluate_likelihood;
	GetOutputFn get_output;
};

template<typename T>
T LikelihoodDynamicLibrary::GetFunction(const char* name)
{
#if PLATFORM_WINDOWS
	FARPROC proc = (FARPROC)GetProcAddress(module, name);
	if (!proc) {
		LOGERROR("GetProcAddress(%s) failed: %u", name, GetLastError());
		return NULL;
	}
	return (T)proc;
#else
	dlerror();
	void* proc = dlsym(library, name);
	if (!proc) {
		LOGERROR("dlsym(%s) failed: %s", name, dlerror());
		return NULL;
	}
	return (T)proc;
#endif
}
