#include "Utils.h"
#include "CSVParser.h"
#include "Data.h"
#include "ParseUtilFunctions.h"

#include <fstream>
#include <boost/regex.hpp>

namespace ModelParser {

Data::Data()
{
}

Data::~Data()
{
}

bool Data::Parse(const std::string& line, unsigned int linenr)
{
	boost::regex data_re("(\\w+)[\\s\\t]{0,}=[\\s\\t]{0,}\"(.+?)\"");
	boost::smatch results;
	if (boost::regex_match(line, results, data_re) && results.size() == 3) {
		Name = results[1];
		Filename = results[2];

		LOG("Loading data file \"%s\"...", Filename.c_str());
		bcm::CSVParser parser;
		if (!parser.Parse(Filename.c_str(), "\t")) {
			return false;
		}
		
		Variables.resize(parser.GetNumColumns());
		for (size_t vi = 0; vi < Variables.size(); vi++) {
			Variables[vi] = parser.GetColumnName(vi);
		}
		
		Observations.resize(parser.GetNumRows());
		for (size_t oi = 0; oi < Observations.size(); oi++) {
			Observations[oi] = parser.GetRowName(oi);
		}

		Values.resize(Observations.size(), Variables.size());
		for (size_t vi = 0; vi < Variables.size(); vi++) {
			for (size_t oi = 0; oi < Observations.size(); oi++) {
				Values(oi,vi) = parser.GetEntry(oi, vi);
			}
		}
	} else {
		LOGERROR("Syntax error in line %u: \"%s\", data", linenr, line.c_str());
		return false;
	}

	return true;
}

void Data::WriteHeader(std::ofstream& ofs) const
{
	ofs << "\tstatic const double data_" << Name << "[" << std::to_string((uint64)Observations.size()) << "][" << std::to_string((uint64)Variables.size()) << "];\n";
}

void Data::WriteSource(std::ofstream& ofs, std::string class_name) const
{
	ofs << "const double " << class_name << "::data_" << Name << "[" << std::to_string((uint64)Observations.size()) << "][" << std::to_string((uint64)Variables.size()) << "] = {\n";
	for (size_t oi = 0; oi < Observations.size(); oi++) {
		ofs << "\t{";
		for (size_t vi = 0; vi < Variables.size(); vi++) {
			char buf[50];
			if (Values(oi,vi) == Values(oi,vi)) {
				snprintf(buf, 50, "%6g", Values(oi,vi));
			} else {
				snprintf(buf, 50, "std::numeric_limits<double>::quiet_NaN()");
			}
			ofs << buf << ",";
		}
		ofs << "},\n";
	}
	ofs << "};\n";
}

bool Data::GetVariableIndex(const std::string& varname, size_t& ix) const
{
	for (size_t vi = 0; vi < Variables.size(); vi++) {
		if (Variables[vi] == varname) {
			ix = vi;
			return true;
		}
	}
	return false;
}

}
