#pragma once

#include "Data.h"
#include "Likelihood.h"
#include "Relation.h"
#include "Variable.h"

namespace ModelParser {

class Model
{
public:
	Model();
	~Model();

	bool Parse(const std::string& inputfn);
	bool WriteHeader(const std::string& outputpath);
	bool WriteSource(const std::string& outputpath);
	bool WriteVariableFile(const std::string& outputpath);
	bool WriteCMakeFile(const std::string& outputpath, const std::string& bcm_path);

	const Variable* FindVariable(const std::string& name) const;
	const Data* FindData(const std::string& name) const;
	std::string GetClassName() const;

private:
	enum ESection {
		S_Constants,
		S_Data,
		S_Variables,
		S_Priors,
		S_Likelihood,
		S_Invalid,
	};

	Variable* FindVariable(const std::string& name);
	Data* FindData(const std::string& name);
	ESection GetSection(const std::string& str);
	inline std::string GetIndentString() { return std::string(Evaluation_CurrentIndentation, '\t'); }
	
	std::string ModelName;

	std::vector<Data> Datasets;
	std::vector<Variable> Variables;
	std::vector<Relation> Relations;
	Likelihood* likelihood;
	std::map<std::string, std::string> Constants;
	std::vector<std::string> EvaluationCode;

	int Evaluation_CurrentIndentation;
	std::set<std::string> Evaluation_LoopVariables;
};

}
