#pragma once

#include "LinearSolver.h"

#if ODE_SINGLE_PRECISION
	typedef float OdeReal;
	typedef Eigen::VectorXf OdeVectorReal;
	typedef Eigen::MatrixXf OdeMatrixReal;
#else
	typedef double OdeReal;
	typedef Eigen::VectorXd OdeVectorReal;
	typedef Eigen::MatrixXd OdeMatrixReal;
#endif

class CVODESolver
{
public:
	typedef boost::function<bool (OdeReal, const OdeReal*, OdeReal*, void*)> TDeriviativeFunction;
	typedef boost::function<bool (OdeReal, const OdeReal*, const OdeReal*, OdeReal**, void*)> TJacobianFunction;

	CVODESolver();
	~CVODESolver();

	void SetLinearSolver(LinearSolver* solver);
	bool Initialize(size_t N, void* user);
	bool SetTolerance(Real relative, Real absolute);
	bool SetTolerance(Real relative, VectorReal absolute);
	void SetUserData(void* user);

	void SetDerivativeFunction(TDeriviativeFunction f);
	void SetJacobianFunction(TJacobianFunction f);

	bool Simulate(const Real* initial_conditions, const std::vector<Real>& timepoints, MatrixReal& output);

	void DumpStatistics(const char* filename);

	int cvode_rhs_fn(OdeReal t, OdeReal* y, OdeReal* ydot);
	int cvode_jac_fn(OdeReal t, OdeReal* y, OdeReal* ydot, OdeReal** jac);
	int cvode_solver_init();
	int cvode_solver_setup(void* cvode_mem, int convfail, OdeReal* ypred, int* jcurPtr);
	int cvode_solver_solve(void* cvode_mem, OdeReal* b);
	void cvode_solver_free();

private:
	void* cvode_mem;
	size_t N;
	void* y;
	void* interpolate_y;
	void* user_data;
	LinearSolver* linear_solver;

	TDeriviativeFunction derivative;
	TJacobianFunction jacobian;

	std::vector<size_t> step_counts;
	std::vector<size_t> failed_step_counts;
};
