#pragma once

#include "NestedSamplingStrategy.h"

namespace bcm {

class Prior;

class NestedSamplingStrategyMultiNest : public NestedSamplingStrategy
{
public:
	NestedSamplingStrategyMultiNest(SamplerNested* sampler);
	virtual ~NestedSamplingStrategyMultiNest();
	
	virtual bool Initialize(Real prior_volume_estimate, RNG* rng);
	virtual bool UpdateRemoveSample(const Sample* removed_sample, Real prior_volume_estimate, bool& reset, RNG* rng);
	virtual bool UpdateAddSample(const Sample* added_sample, RNG* rng);
	virtual bool GetNewSample(Real min_llh, Sample* sample, RNG* rng, size_t threadix) const;

private:
	struct Cluster
	{
		Cluster(size_t nvar);
		~Cluster();

		void CalculateMean(const std::vector<VectorReal>& unit_samples);
		void CalculateCovariance(const std::vector<VectorReal>& unit_samples);
		void CalculateBoundingEllipsoid(const std::vector<VectorReal>& unit_samples);
		void Recalculate(const std::vector<VectorReal>& unit_samples, Real minimum_volume, RNG* rng);
		void Rescale(Real minimum_volume, Real n);

		size_t nvar;
		std::vector<const Sample*> samples;	// This vector is continuously updated when samples are removed and added
		std::vector<size_t> sample_ixs;		// This vector is only updated during Recalculate()
		VectorReal mean;
		MatrixReal covariance;
		MatrixReal inv_covariance;
		MatrixReal transformation;
		Real scaling;
		Real volume;
		
		Cluster* parent;
		Cluster* children[2];
	};

	bool Recalculate(Real minimum_volume, RNG* rng);
	void GatherLeafs(Cluster* cluster);

	// Settings
	Real enlargement;
	size_t full_update_steps;

	// Run-time variables
	size_t steps_since_update;
	Cluster* root;
	std::vector<Cluster*> leafs;
	Real total_volume;
	std::vector<VectorReal> unit_samples;	// This vector is only updated during Recalculate()
};

}
