#pragma once

#ifdef BCM_INCLUDE_NETCDF

#include "NetCDFDataFile.h"

namespace bcm {

class AdditionalOutputInterfaceNetCDF
{
public:
	virtual bool SetupAdditionalOutput(NetCDFDataFile* file, size_t sample_count) = 0;
	virtual void HandleAdditionalOutput(NetCDFDataFile* file, size_t sample_ix, const Real* values) = 0;
};

class OutputHandlerNetCDF
{
public:
	OutputHandlerNetCDF();
	~OutputHandlerNetCDF();
	
	void SetFile(const std::string& filename);
	void SetAdditionalOutput(AdditionalOutputInterfaceNetCDF* additional_output);
	bool Initialize(size_t sample_count, std::vector<std::string>& variables);
	void Close();

	void HandleSample(const Real* values, Real lposterior, Real llh, Real lprior, Real weight, void*);
	void HandleAdditionalOutput(const Real* values, void*);
	
	boost::function<void (const Real* values, Real lposterior, Real llh, Real lprior, Real weight, void* user)> GetSampleFunctionPointer();
	boost::function<void (const Real* values, void* user)> GetAdditionalOutputFunctionPointer();

private:
	std::string Filename;
	NetCDFDataFile NetCDFFile;
	size_t NumVariables;
	size_t sample_ix;
	AdditionalOutputInterfaceNetCDF* AdditionalOutput;
};

}

#endif
