#pragma once

namespace bcm {

class RNG;
class VariableSet;

class Prior
{
public:
	Prior(VariableSet* varset);
	virtual ~Prior();

	static Prior* Create(const std::string& filename, VariableSet* varset);
		
	virtual bool LoadFromXML(const boost::property_tree::ptree& xml_node) { return false; }
	virtual bool EvaluateLogPDF(size_t threadix, const Real* values, Real& logp) const = 0;
	virtual bool Sample(Real* values, RNG* rng) const = 0;

	inline Real GetLowerBound(size_t i) const { return LowerBounds[i]; }
	inline Real GetUpperBound(size_t i) const { return UpperBounds[i]; }

	// Required for MH/PT sampler:
	virtual bool EvaluateMarginalVariance(size_t i, Real& var) const { return false; }

	// Required for Nested sampler:
	virtual bool EvaluateMarginalCDF(size_t i, const Real value, Real& p) const { return false; }
	virtual bool EvaluateMarginalQuantile(size_t i, const Real p, Real& value) const { return false; }

protected:
	bool LoadTransformationsFromXML(const boost::property_tree::ptree& xml_node);
	bool ApplyTransformations(const Real* values, VectorReal& v) const;
	bool CorrectProbabilityForTransform(Real& logp, const Real* values, VectorReal& v) const;
	bool ApplyInverseTransformations(Real* v) const;

	VariableSet* varset;
	std::vector<Real> LowerBounds;
	std::vector<Real> UpperBounds;
	
	enum ETransform {
		Transform_Invalid,
		Transform_None,
		Transform_Log,
		Transform_Logit,
		Transform_LogitScale,
	};
	std::vector<ETransform> transforms;
	std::vector<VectorReal> transform_parameters;
};

}
