#include "Utils.h"
#include "ProgressIndicator.h"

namespace bcm {

ProgressIndicator::ProgressIndicator()
	: current_run_time(0.0)
	, expected_run_time(0.0)
	, expected_remaining_time(0.0)
	, expected_remaining_time_ema(0.0)
{
}

ProgressIndicator::~ProgressIndicator()
{
}

void ProgressIndicator::NotifyStart()
{
	timer.Start();
}

void ProgressIndicator::NotifyStop()
{
	OnStop();
}

void ProgressIndicator::UpdateInfo(const char* name, const std::string& value)
{
	current_info[name] = value;
}

void ProgressIndicator::ReportProgress(Real progress)
{
	current_progress = progress;
	current_run_time = std::max(1e-5, timer.GetElapsedSeconds());
	if (progress <= 0) {
		progress = 1e-5;
	}
	expected_run_time = current_run_time / progress;
	expected_remaining_time = current_run_time * (1.0 - progress) / progress;
	expected_remaining_time_ema += (expected_remaining_time - expected_remaining_time_ema) * 0.1;

	if (expected_remaining_time > (Real)std::numeric_limits<uint64>::max()) {
		eta = "infinite";
	} else {
		uint64 seconds = (uint64)expected_remaining_time;
		uint64 minutes = seconds / 60;
		seconds -= minutes * 60;
		uint64 hours = minutes / 60;
		minutes -= hours * 60;
		uint64 days = hours / 24;
		hours -= days * 24;

		char buf[32];
		snprintf(buf, sizeof(buf), "%llud %02lluh %02llum %02llus", days, hours, minutes, seconds);
		eta = buf;
	}

	OnUpdate();
}

}
