#pragma once

#include "RNG.h"
#include "Sampler.h"
#include "SamplerPTChain.h"

namespace bcm {

class SamplerPT : public Sampler
{
public:
	SamplerPT(size_t num_threads);
	virtual ~SamplerPT();

	virtual size_t GetNumSamples() { return sampling_period; }

private:
	virtual bool ParseImpl(const boost::program_options::variables_map& vm);
	virtual bool InitializeImpl();
	virtual void UninitializeImpl();
	virtual ERunResult RunImpl();
	void DoExchangeMove(bool burnin, size_t sampleIx);
	bool DoMutateMove(bool burnin);
	void EmitSample(bool burnin, size_t sampleIx);
	void LogStatistics();
	void AdaptTemperatures();
	void CalculateMarginalLikelihood();
	void UpdateProgress(Real p);

	// Settings
	size_t num_chains;
	size_t burnin_period;
	size_t sampling_period;
	size_t use_every_nth;
	size_t num_mcmc_moves_per_iteration;
	Real min_temperature;
	std::string temperatures_file;
	Real initial_temperature_power;

	Real exchange_probability;
	size_t adapt_proposal_samples;
	Real proposal_scaling_learning_rate;
	size_t proposal_scaling_ema_period;
	Real target_acceptance_rate;

	size_t adapt_temperatures_times;
	size_t adapt_temperatures_period;

	enum ProposalType {
		PT_Scalar,
		PT_FullVector,
		PT_AutoBlock,
	};
	ProposalType proposal_type;
	
	// Run-time variables
	RNG* rng;
	VectorReal temperatures;
	VectorReal thermodynamic_integrand;
	VectorReal thermodynamic_integrand_variance;
	std::vector<SamplerPTChain> chains;
	std::vector<VectorReal> samples_lposterior;

	// Run-time variables for feedback optimization
	VectorReal up_count;
	VectorReal down_count;
	Eigen::VectorXi labels;
	VectorReal up_labeled_time;
	VectorReal down_labeled_time;
	std::vector<Real> up_trip_time;
	std::vector<Real> down_trip_time;

	friend class SamplerPTChain;
};

}
