#include "Utils.h"
#include "SBMLAssignmentRule.h"

SBMLAssignmentRule::SBMLAssignmentRule()
	: rate_law(NULL)
	, evaluate_rate_law(NULL)
{
}

SBMLAssignmentRule::~SBMLAssignmentRule()
{
	bcm::safe_delete(rate_law);
	bcm::safe_delete(evaluate_rate_law);
}
	
bool SBMLAssignmentRule::Initialize(const Rule* rule, const SBMLModel* model)
{
	if (!rule->isAssignment()) {
		LOGERROR("SBML rule is not an assignment");
		return false;
	}

	rate_law = rule->getMath()->deepCopy();
	TargetVariable = rule->getVariable();

	return true;
}

bool SBMLAssignmentRule::SetSpeciesVector(const std::vector<std::string>& species_ids)
{
	// Check if any of the named AST nodes map to a species
	MapRateLawSpecies(species_ids, rate_law);
	return true;
}

bool SBMLAssignmentRule::SetVariableSet(const bcm::VariableSet* variables)
{
	// Check if any of the named AST nodes map to a variable
	MapRateLawParameters(variables, rate_law);
	return true;
}

bool SBMLAssignmentRule::PostInitialize()
{
	bcm::safe_delete(evaluate_rate_law);
	return SBMLRatelawElement::Generate(rate_law, species_index_map, parameter_index_map, &evaluate_rate_law);
}

bool SBMLAssignmentRule::Calculate(const Real* species, const Real* parameters, Real* value) const
{
	ASSERT(value);
	*value = evaluate_rate_law->Evaluate(species, parameters);
	return true;
}

void SBMLAssignmentRule::MapRateLawSpecies(const std::vector<std::string>& species_ids, const ASTNode* node)
{
	if (node->getType() == AST_NAME) {
		const char* name = node->getName();
		for (size_t i = 0; i < species_ids.size(); i++) {
			if (species_ids[i] == name) {
				species_index_map[name] = i;
			}
		}
	} else {
		for (unsigned int i = 0; i < node->getNumChildren(); i++) {
			MapRateLawSpecies(species_ids, node->getChild(i));
		}
	}
}

void SBMLAssignmentRule::MapRateLawParameters(const bcm::VariableSet* variables, const ASTNode* node)
{
	if (node->getType() == AST_NAME) {
		const char* name = node->getName();
		for (size_t i = 0; i < variables->GetNumVariables(); i++) {
			if (variables->GetVariableName(i) == name) {
				parameter_index_map[name] = i;
			}
		}
	} else {
		for (unsigned int i = 0; i < node->getNumChildren(); i++) {
			MapRateLawParameters(variables, node->getChild(i));
		}
	}
}
