#pragma once

#include "NetCDFDataFile.h"
#include "VariableSet.h"

class ExperimentalCondition;
class LikelihoodPart;
class SBMLModel;

class LikelihoodExperiment
{
public:
	LikelihoodExperiment(size_t numthreads);
	~LikelihoodExperiment();

	bool Load(const boost::property_tree::ptree& xml_node, const bcm::NetCDFDataFile* datafile);
	bool SetVariableSet(const bcm::VariableSet* variables);
	bool Initialize();
	void SetNumModelSpecies(size_t n);
	bool SetupAdditionalOutput(bcm::NetCDFDataFile* file, SBMLModel* model, size_t sample_count);
	void HandleAdditionalOutput(bcm::NetCDFDataFile* file, SBMLModel* model, size_t sample_ix, const Real* values, size_t& value_ix);
	bool EvaluateLogProbability(size_t threadix, const Real* values, SBMLModel* model, Real& logp);
	bool GetOutput(size_t threadix, Real* output_values, size_t& numvalues);

private:
	size_t num_threads;
	std::string Name;
	std::vector<ExperimentalCondition*> Conditions;
	std::vector<LikelihoodPart*> Parts;
	std::vector<Real> RequiredTimepoints;
	std::vector< std::vector<VectorReal> > SimulatedTrajectories;
	std::vector<std::string> SaveOutputs;
};
