#pragma once

#include "Likelihood.h"
#include "OutputHandlerNetCDF.h"
#include "VariableSet.h"
#include <boost/multi_array.hpp>

class SignalingNetwork;
class SBMLPDExperiment;

class SBMLPDLikelihood : public bcm::Likelihood, public bcm::AdditionalOutputInterfaceNetCDF
{
public:
	SBMLPDLikelihood(size_t numthreads);
	~SBMLPDLikelihood();
	
	bool SetVariableSet(const bcm::VariableSet* varset);
	bool Initialize(const std::string& likelihood_file);
	bool SetupAdditionalOutput(bcm::NetCDFDataFile* file, size_t sample_count);
	void HandleAdditionalOutput(bcm::NetCDFDataFile* file, size_t sample_ix, const Real* values);

	virtual bool EvaluateLogProbability(size_t threadix, const Real* values, Real& logp);
	virtual bool GetOutput(size_t threadix, Real* output_values, size_t& numvalues);

private:
	bool CalculateModel(VectorReal& activities, const Real* values) const;
	
	inline Real GetValue(const Real* values, size_t varix) const {
		ASSERT(varix < VarSet->GetNumVariables());
		return values[varix];
	}

	size_t numthreads;
	const bcm::VariableSet* VarSet;

	SignalingNetwork* network;

	std::vector<SBMLPDExperiment*> Experiments;
};
